/*
Copyright (c) 2005, Alan Snelson
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided
that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the
following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions
and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the Wave2 Limited nor the names of its contributors may be used to
endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE
, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

var feed = "http://del.icio.us/rss";	
var last_updated = 0;
var xml_request = null;

if (window.widget)
{
	widget.onshow = show;
	widget.onremove = onremove;
}

function getLocalizedString (key)
{
	try {
		var ret = localizedStrings[key];
		if (ret === undefined)
			ret = key;
		return ret;
	} catch (ex) {}

	return key;
}

var everBeenCalled = false;
function specialFirstLoad ()
{
	doPreloadSetup();
	if (!everBeenCalled)
	{
		onshow();
		everBeenCalled = true;
	}
	
}

// called before the page is loaded, get prefs, do localization, etc
function doPreloadSetup()
{	
	scrollerInit(document.getElementById("myScrollBar"), document.getElementById("myScrollTrack"), document.getElementById("myScrollThumb"));
	calculateAndShowThumb(document.getElementById("contents"));
	//document.getElementById('ziplabel').innerText = getLocalizedString('City, State or ZIP Code:');
	if(window.widget) 
	{ 
    	var username = widget.preferenceForKey("username"); 
    	if (username && username.length > 0) 
    	{ 
        	feed = "http://del.icio.us/rss/" + username;
			document.getElementById('username').value = username;
    	} 
	} 
	document.getElementById('copyright').innerText = "Copyright 2005 Wave2 Limited";
	createGenericButton(document.getElementById('done'), getLocalizedString('Done'), showfront);
}

//------------------------------------------------------------------------------
//  show - post a request to get DUB Messages Overall RSS topics when showing 
//    the widget.
//  Space requests by at least 15 minutes to avoid hitting the server too often. 
//  Also, note that you should cancel any outstanding request before posting 
//  the new one. 
//------------------------------------------------------------------------------

function show ()
{
	var now = (new Date).getTime();
	
	// only check if 15 minutes have passed or user changes url
	if ((document.getElementById("back").style.display == "block") || ((now - last_updated) > 900000))
	{
		if (xml_request != null)
		{
			xml_request.abort();
			xml_request = null;
		}
		xml_request = new XMLHttpRequest();
		xml_request.onload = function(e) {xml_loaded(e, xml_request);}
		xml_request.overrideMimeType("text/xml");
		xml_request.open("GET", feed);
		xml_request.setRequestHeader("Cache-Control", "no-cache");
		xml_request.send(null);
    }
}


//-----------------------------------------------------------------------------
// findChild - scan the children of a given DOM element for a node matching
// nodeName; much more efficient than the standard DOM methods
// (getElementsByTagName, etc) if you know what you're looking for
//-----------------------------------------------------------------------------

function findChild (element, nodeName)
{
	var child;
	
	for (child = element.firstChild; child != null; child = child.nextSibling)
	{
		if (child.nodeName == nodeName)
			return child;
	}
	
	return null;
}

//----------------------------------------------------------------------------
// xml_loaded - extract the content of Apple RSS Hot News feed and place the
// items data into a results array: extract the title, link and publication 
// date for each item. 
//----------------------------------------------------------------------------

function xml_loaded (e, request) 
{
	//alert (request.responseText);
	xml_request = null;
	if (request.responseXML)
	{		
		// Get the top level <rdf> element 
		var rss = findChild(request.responseXML, 'rdf:RDF');		
		
		var results = new Array;
		
		// Get all item elements subordinate to the channel element
		// For each element, get title, link and publication date. 
		// Note that all elements of an item are optional. 
		for( var item = rss.firstChild; item != null; item = item.nextSibling)
		{
			if( item.nodeName == 'item' ) 
			{
				var title = findChild (item, 'title');
			
				// we have to have the title to include the item in the list 
				if( title != null ) 
				{
					var link = findChild (item, 'link');
					results[results.length] = {title:title.firstChild.data, 
						link:(link != null ? link.firstChild.data : null)
				   };
				}
			}
		}
		
		// sort by date
		results.sort (compFunc);
		
		// copy title and date into rows for display. Store link so it can be used when user
		// clicks on title
		nItems = results.length;
		
		//update notification pane
		if(nItems > 0){
			var notify = document.getElementById('notify');
			notify.innerText = getLocalizedString(nItems + " Bookmarks Found.");
			showfront();
		}else{
			var notify = document.getElementById('notify');
			notify.innerText = getLocalizedString("No Bookmarks Found!");
		}
		var even = true;		
				
		for (var i = 0; i < nItems; ++i)
		{
			var item = results[i];
			var row = createRow (item.title, item.link, even);
			even = !even;
			
			contents.appendChild (row);
		}

		// update the scrollbar so scrollbar matches new data
		calculateAndShowThumb(document.getElementById("contents"));
		
		// set last_updated to the current time to keep track of the last time a request was posted
		last_updated = (new Date).getTime();
	}

}

//-----------------------------------------------------------------------------
// sortFunc - compare function used for sorting dates
//-----------------------------------------------------------------------------

function compFunc (a, b)
{
	if (a.date < b.date)
		return 1;
	else if (a.date > b.date)
		return -1;
	else
		return 0;
}


var flipAnimation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, element:null, timer:null};
var flipShown = false;

function limit_3 (a, b, c) {
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
    return from + (to - from) * ease;
}

var usernameValidated = false;
function usernameChanged()
{
	var notify = document.getElementById('notify');
	notify.innerText = getLocalizedString('Retrieving Bookmarks...');
	notify.style.display = "block";
	var newUsername = document.getElementById('username').value;
	feed = "http://del.icio.us/rss/" + newUsername;
	var rsslist = document.getElementById("contents");
    while (rsslist.childNodes.length > 0) {
        rsslist.removeChild(rsslist.childNodes[0]);
    }
	show();
	if (validFeed == false)
	{
		document.getElementById('username').value = "False";
	}
	usernameValidated = true;
}

function usernameTyping (event)
{
	usernameValidated = false;
}

function usernameKeyPress (event)
{
	switch (event.keyCode)
	{
		case 13: // return
		case 3:  // enter
		case 9:  // tab
			if (!usernameValidated)
			{
				usernameChanged();
			}
			break;
	}
}

function mousemove (event)
{
	if (!flipShown)
	{
		// fade in the flip widget
		if (flipAnimation.timer != null)
		{
			clearInterval (flipAnimation.timer);
			flipAnimation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13; // set it back one frame
		
		flipAnimation.duration = 500;
		flipAnimation.starttime = starttime;
		flipAnimation.element = document.getElementById ('flip');
		flipAnimation.timer = setInterval ("flipAnimate();", 13);
		flipAnimation.from = flipAnimation.now;
		flipAnimation.to = 1.0;
		flipAnimate();
		flipShown = true;
	}
}

function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the flip widget
		if (flipAnimation.timer != null)
		{
			clearInterval (flipAnimation.timer);
			flipAnimation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13; // set it back one frame
		
		flipAnimation.duration = 500;
		flipAnimation.starttime = starttime;
		flipAnimation.element = document.getElementById ('flip');
		flipAnimation.timer = setInterval ("flipAnimate();", 13);
		flipAnimation.from = flipAnimation.now;
		flipAnimation.to = 0.0;
		flipAnimate();
		flipShown = false;
	}
}

function flipAnimate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
		
	
	T = limit_3(time-flipAnimation.starttime, 0, flipAnimation.duration);
	
	if (T >= flipAnimation.duration)
	{
		clearInterval (flipAnimation.timer);
		flipAnimation.timer = null;
		flipAnimation.now = flipAnimation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / flipAnimation.duration));
		flipAnimation.now = computeNextFloat (flipAnimation.from, flipAnimation.to, ease);
	}
	
	flipAnimation.element.style.opacity = flipAnimation.now;
}

function showbackside (event)
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	isShowingFront = false;
	if (window.widget)
		widget.prepareForTransition("ToBack");
	
	front.style.display="none";
	back.style.display="block";
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);	
	
	document.getElementById('fliprollie').style.display = 'none';
		
	event.stopPropagation();
	event.preventDefault();
}

function showfront ()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	isShowingFront = true;
	
	if (window.widget)
		widget.prepareForTransition("ToFront");
	
	front.style.display="block";
	back.style.display="none";
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);
	
	if(window.widget) 
	{ 
   		widget.setPreferenceForKey(document.getElementById('username').value ,"username"); 
	} 

}

function onremove ()
{
	if (window.widget)
	{
		widget.setPreferenceForKey (null, "username");
	}
}

function limit_3 (a, b, c) {
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
    return from + (to - from) * ease;
}

//----------------------------------------------------------------------------
// createRow - add data to the next row in the widget body. Rows have 
// alternating (light and dark backgound). The title and date as displayed 
// for each item. The link is used  when the user clicks on a RSS title. 
//----------------------------------------------------------------------------
function showImage(img){ return (function(){ img.style.display='inline' }) }
function createRow (title, link, even)
{
	var row = document.createElement ('div');
	row.setAttribute ('class', 'row ' + (even ? 'light' : 'dark'));
	
	var title_div = document.createElement ('div');
	title_div.innerText = title;
	title_div.setAttribute ('class', 'title');
	if (link != null)
	{			
		var favicon_div = document.createElement('div');
		favicon_div.setAttribute ('class', 'favicon');
		var img = document.createElement('img');
		img.style.display = 'none'
    	img.height = img.width = 16;
   		img.src = link.split('/').splice(0,3).join('/')+'/favicon.ico';
		img.onload = showImage(img);
		favicon_div.appendChild(img);
		title_div.setAttribute ('the_link', link);
		title_div.setAttribute ('onclick', 'clickOnTitle (event, this);');
	}
	row.appendChild (favicon_div);
	row.appendChild (title_div);
	
	return row;	
}

function enterflip(event)
{
	document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event)
{
	document.getElementById('fliprollie').style.display = 'none';
}

//----------------------------------------------------------------------------
// clickOnTitle - take user to the RSS link when she clicks on an article's title.
//----------------------------------------------------------------------------
function clickOnTitle (event, div)
{
	if (window.widget)
	{
		widget.openURL (div.the_link);
	} else document.location = div.the_link;
}

function feedTitleClick(event)
{
	if (window.widget)
	{
			widget.openURL ("http://del.icio.us");
	} else document.location = "http://del.icio.us";
}

//----------------------------------------------------------------------------
// clickOnFeedTitle - take user to the feed main web page when she clicks 
// on the widget title. 
//----------------------------------------------------------------------------

function copyrightClick(event)
{
	if (window.widget)
	{
			widget.openURL ("http://www.wave2.co.uk");
	} else document.location = "http://www.wave2.co.uk";
}
